@@ -186,7 +186,6 @@ ini_cfg_create_config(
 {
     uint32_t err = 0;
     PCONFIG_INI pConfig = NULL;
-    char szPath[1024];
 
     if (!ppConfig)
     {
@@ -197,11 +196,6 @@ ini_cfg_create_config(
     err = netmgr_alloc(sizeof(CONFIG_INI), (void*)&pConfig);
     bail_on_error(err);
 
-    sprintf(szPath, "/tmp/%s", tmpnam(NULL));
-
-    err = netmgr_alloc_string(szPath, &pConfig->pszPath);
-    bail_on_error(err);
-
     *ppConfig = pConfig;
 
 cleanup:
@@ -214,6 +208,10 @@ error:
     {
         *ppConfig = NULL;
     }
+    if (pConfig)
+    {
+        ini_cfg_free_config(pConfig);
+    }
 
     goto cleanup;
 }
@@ -539,7 +537,8 @@ ini_cfg_delete_key(
     )
 {
     uint32_t err = 0;
-    PKEYVALUE_INI pCursor = NULL;
+    PKEYVALUE_INI *pCursor = NULL;
+    PKEYVALUE_INI pCandidate = NULL;
 
     if (!pSection || !pszKey || !*pszKey)
     {
@@ -547,29 +546,22 @@ ini_cfg_delete_key(
         bail_on_error(err);
     }
 
-    pCursor = pSection->pKeyValue;
-    while (pCursor)
+    pCursor = &pSection->pKeyValue;
+    while (*pCursor)
     {
-        PKEYVALUE_INI pCandidate = NULL;
-
-        if (!strcmp(pCursor->pszKey, pszKey))
+        if (!strcmp((*pCursor)->pszKey, pszKey))
         {
-            pCandidate = pCursor;
-
-            if (pCursor == pSection->pKeyValue)
-            {
-                pSection->pKeyValue = pCursor->pNext;
-            }
+            pCandidate = *pCursor;
+            *pCursor = pCandidate->pNext;
+            break;
         }
+        pCursor = &(*pCursor)->pNext;
+    }
 
-        pCursor = pCursor->pNext;
-
-        if (pCandidate)
-        {
-            pCandidate->pNext = NULL;
-
-            ini_cfg_free_keyvalue(pCandidate);
-        }
+    if (pCandidate)
+    {
+        pCandidate->pNext = NULL;
+        ini_cfg_free_keyvalue(pCandidate);
     }
 
 error:
@@ -602,7 +594,7 @@ ini_cfg_save(
 
     sprintf(pszTmpPath, "%s%s", pszPath, pszSuffix);
 
-    fp = fopen(pszTmpPath, "w");
+    fp = fopen(pszTmpPath, "w+");
     if (!fp)
     {
         err = errno;
@@ -613,17 +605,30 @@ ini_cfg_save(
     {
         PKEYVALUE_INI pKeyValue = pSection->pKeyValue;
 
-        fprintf(fp, "\n[%s]\n", pSection->pszName);
+        if(fprintf(fp, "\n[%s]\n", pSection->pszName) < 0)
+        {
+            err = EBADF;
+            bail_on_error(err);
+        }
 
         for (; pKeyValue; pKeyValue = pKeyValue->pNext)
         {
-            fprintf(fp, "%s=%s\n", pKeyValue->pszKey, pKeyValue->pszValue);
+            if(fprintf(fp, "%s=%s\n", pKeyValue->pszKey, pKeyValue->pszValue) < 0)
+            {
+                err = EBADF;
+                bail_on_error(err);
+            }
         }
     }
 
     fclose(fp);
     fp = NULL;
 
+    if (chmod(pszTmpPath, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH) != 0)
+    {
+        bail_on_error(errno);
+    }
+
     if (rename(pszTmpPath, pszPath) < 0)
     {
         err = errno;
